##------------------------------------------------------------------------------
##
##              TOOL FUNCTION
##
##------------------------------------------------------------------------------
##credit goes to R package mvtnorm
rmvnorm <- function (n, mean, sigma) {
    ev     <- eigen(sigma, symmetric = TRUE);
    retval <- ev$vectors %*%
              diag(sqrt(ev$values), length(ev$values)) %*%
              t(ev$vectors);
    retval <- matrix(rnorm(n * ncol(sigma)), nrow = n) %*% retval
    retval <- sweep(retval, 2, mean, "+")
    retval
}


##------------------------------------------------------------------------------
##
##              HDPMN FITTING
##
##------------------------------------------------------------------------------
hdpreg.WriteInitP <- function(data.y, data.x, data.study,
                              is.mdp, is.bin,
                              n.iter=6000, n.discard=3000, n.batch=10,
                              verbose=3,  # 0=no comments 3comments
                              mcmc.eps=1, # indicator for sampling epsilon
                              eps=1, 	  # prob idiosyncratic cluster
                              ae=1.0,     # p(eps) ~ pe0*I(0) + pe1*I(1) + (1-pe0-pe1)*Be(ae,be)
                              be=1.0,
                              pe1=0.1,
                              pe0=0.1,
                              m.prior=1,     # resample m: m.prior <- 0 or 1 required.
                              B.prior=1,     # resample B: B.prior <- 0 or 1 required.
                              S.prior=1,     # resample S
                              alpha.prior=1, # resample alpha
                              az=0.01,       # prior for sigz2
                              bz=0.01,
                              nu0=0,         # prior for nu
                              R0=100,
                              q=10,         # prior for var(beta)
                              R1=10,
                              a0=0,          # prior for E(beta)
                              b0=100,
                              aB=1,          # prior for var(G0)=B
                              bB=1,
                              aa=1,          # prior for alpha
                              ba=1) {


    ##regular regression
    if (!is.mdp) {
        ##add intercept
        data.x <- cbind(rep(1, nrow(data.x)), data.x);
    }

    write.table(data.y, file="data-y.giis",
                sep=" ", quote=F,   row.names=F, col.names=F);
    write.table(data.x, file="data-x.giis",
                sep=" ", quote=F, row.names=F, col.names=F);

    npatients <- length(data.study);
    nstudies  <- length(unique(data.study));
    seed1     <- trunc(10000000*runif(1)); # r.v. seeds
    seed2     <- trunc(10000000*runif(1));


    ##----pars for DP mixture r.e. prior
    px <- ncol(data.x) # dimension of random eff (incl cov's)
    n.predupdate <- n.batch      # modified by cg on 9/27 #old value 20
    n.pi         <- 1

    sig.z.init <- 1.0	# initial value forsigma, residual variance.  Set to 1 for probit.
    beta.init  <- array(0, dim=c(nstudies, px)); #init for beta
    nu.init    <- rep(0, px);    # hyperpiror mean for beta
    nu0        <- rep(nu0, px);
    R0         <- R0*diag(px);
    Sbinv.init <- diag(px);      # initial value and prior mean for S
    m.init     <- 0; # initial values for m
    B.init     <- 1; # initial value for B in G0(mu)=N(m,B)
    alpha      <- 1; # initial value for total mass alpha i.e. M
    R1         <- R1*diag(px);
    k0         <- nstudies+1;  # initial number of clusters


    ##do not use scientific notation
    old.scipen <- getOption('scipen');
    options(scipen=999);

    ## write init file
    out <- file("init.giis",open="w");
    cat("data-npatients ",npatients,"\n",
        "data-nstudies ", nstudies, "\n",
        "model-p1 ", px, "\n",
        "data-study ", format(data.study), "\n",
        "data-N ", npatients, "\n",
        "data-bin ", as.integer(is.bin), "\n", ##wheter it is binary data
        "mcmc-niter ", n.iter, "\n",
        "mcmc-npi ", n.pi, "\n",
        "mcmc-nbatch ", n.batch, "\n",
        "mcmc-ndiscard ",n.discard, "\n",
        "mcmc-verbose ", verbose, "\n",
        "mcmc-seed1 ", seed1, "\n",
        "mcmc-seed2 ", seed2, "\n",
        "mcmc-eps ", mcmc.eps, "\n",
        "mcmc-npredupdate ", n.predupdate, "\n",
        "model-eps ", eps, "\n",
        "model-ae ", ae,   "\n",
        "model-be ", be,   "\n",
        "model-pe0 ", pe0, "\n",
        "model-pe1 ", pe1, "\n",
        "model-m_prior ", m.prior, "\n",
        "model-B_prior ", B.prior, "\n",
        "model-S_prior ", S.prior, "\n",
        "model-alpha_prior ", alpha.prior, "\n",
        file=out);
    close(out);

    ## write init file
    out <- file("init.mdp", open="w");
    cat(" mdp-reg ", as.integer(is.mdp), ##if no mdp, just a regular bayes regression model
        "\n data-p ",  px,
        "\n init-sig ", sig.z.init,
        "\n par-az ", az,
        "\n par-bz ", bz,
        "\n init-beta \n", rbind( format(beta.init), "\n"),
        "\n par-nu0 ",   format(nu0),
        "\n par-R0 \n", rbind( format(R0), "\n"),
        "\n init-Sbinv \n", rbind( format(Sbinv.init), "\n"),
        "\n par-q ",    format(q),
        "\n par-R1 \n",  rbind( format(R1), "\n"),
        "\n init-m ",   format(m.init),
        "\n par-a0 ",   format(a0),
        "\n par-b0 ",   format(b0),
        "\n init-B ",   format(B.init),
        "\n par-aB ",   format(aB),
        "\n par-bB ",   format(bB),
        "\n init-alpha ", format(alpha),
        "\n par-aalpha ", format(aa),
        "\n par-balpha ", format(ba),
        "\n init-k0 ", k0, "\n",
        file=out);
    close(out);

    ##recover
    options(scipen=old.scipen);
}

##------------------------------------------------------------
##          Entry functio for prediction using R
## Parameters:
##
##------------------------------------------------------------
pred.R <- function(mu.covx, fit.rst, sim.each=10) {

    rst <- rep(list(NULL), 2);
    ##j=0 common study j=1..J studies j=J+1 future study
    for (j in 1:(fit.rst$nstudies+1)) {
        for (r in 0:1) {
            print(paste("---j=",j,"----r=",r, "------",sep=""));
            cur.simu.y      <- pred.simu.y(fit.rst, mu.covx, j, r, sim.each);
            rst[[r+1]][[j]] <- cur.simu.y;
        }
    }
    rst
}

##predict y using pred xbeta and
pred.simu.y <- function(fit.rst, cov.x, j, r, each.n) {

    nstudy  <- fit.rst$nstudies;
    mdp.reg <- fit.rst$mdp.reg;

    ##if regular regression
    if (!mdp.reg) {
        cov.x <- c(1, cov.x);
    }

    ##get xbeta
    if ((nstudy+1) == j| 0 == j) {
        rst.mean <- pred.xbeta.newstudy(cov.x, each.n, fit.rst$nu, fit.rst$sinv);
    } else {
        rst.mean <- pred.xbeta.study.j(cov.x, j, each.n, fit.rst$beta)
    }

    ##get mdp intercept
    if (mdp.reg) {
        ##note sigz2 is counted here
        mdp.mu   <- pred.mdp.intercept(fit.rst$mj, j, r, each.n);
        rst.mean <- rst.mean + mdp.mu;
    }

    ##get sigz2
    rst.sigz2  <- rep(fit.rst$ysig[,2], each=each.n);
    ##post index
    post.inx <- rep(fit.rst$ysig[,1], each=each.n);

    ##simu y
    rst.latent <- rnorm(length(rst.sigz2), rst.mean, sqrt(rst.sigz2));

    ##
    if (fit.rst$is.bin) {
        ##binary data
        rst.y <- as.integer(rst.latent > 0);
    } else {
        rst.y <- rst.latent;
    }

    ##result
    rst <- cbind(rst.y, rst.latent);
    rst
}

##predict patients on a new study
##using nubeta and sbeta
pred.xbeta.newstudy <- function(cov.x, each.n, nubeta, sbetainv) {
    np       <- ncol(nubeta) - 1;
    nposts   <- nrow(nubeta);

    rst.beta <- NULL;
    for (i in 1:nposts) {
        cur.m    <- nubeta[i, -1];
        cur.sinv <- matrix(sbetainv[i, -1], np, np);
        cur.s    <- solve(cur.sinv);
        cur.beta <- rmvnorm(each.n, cur.m, cur.s);
        rst.beta <- rbind(rst.beta, cur.beta);
    }

    rst.xbeta <- apply(rst.beta, 1, function(x) {sum(x*cov.x)});
    rst.xbeta
}

pred.xbeta.study.j <- function(cov.x, j, each.n, beta) {
    pbeta  <- length(cov.x);
    nposts <- nrow(beta);

    rst.xbeta <- NULL;
    for (i in 1:nposts) {
        cur.beta  <- beta[i, 1+pbeta*(j-1)+(1:pbeta)];
        cur.xbeta <- sum(cur.beta * cov.x);
        rst.xbeta <- c(rst.xbeta, rep(cur.xbeta, each.n));
    }

    rst.xbeta
}

##j: study r: wheter study only or base G included
pred.mdp.intercept.single <- function(cur.mj, j, r, each.n) {
    ##find components
    cur.inx <- which(j == cur.mj[,2]);
    if (0 !=j & 0 == r) {
        cur.inx <- c(cur.inx, which(0 == cur.mj[,2]));
    }

    if (1 == length(cur.inx)) {
        cur.smp <- rep(cur.inx, each.n);
    } else {
        ##weights sample
        if (0 == r) {
            ##including F0 eps/(1-eps) * w
            cur.weights <- cur.mj[cur.inx, 3]*cur.mj[cur.inx, 4];
        } else {
            ##fj only
            cur.weights <- cur.mj[cur.inx, 4];
        }
        cur.smp.0   <- rmultinom(each.n, 1, cur.weights);
        cur.smp.1   <- apply(cur.smp.0, 2, function(x) {which(1==x)});
        cur.smp     <- cur.inx[cur.smp.1];
    }

    ##simulation
    cur.mu  <- cur.mj[cur.smp, 5];
    ##for model 3 fix cur.sig as 0
    cur.sig <- sqrt(cur.mj[cur.smp, 6]);

    ##return
    rst <- rnorm(length(cur.mu), cur.mu, cur.sig);
    rst <- cur.mu;
}

pred.mdp.intercept <- function(mj.all, ...) {
    inx    <- unique(mj.all[,1]);
    rst.mu <- NULL;
    for (i in 1:length(inx)) {
        cur.inx <- which(inx[i] == mj.all[,1]);
        cur.mj  <- mj.all[cur.inx,];
        cur.rst <- pred.mdp.intercept.single(cur.mj, ...);
        rst.mu  <- c(rst.mu, cur.rst);
    }

    rst.mu
}


##------------------------------------------------------------
##                  MAIN PROGRAM
## Parameters:
##         (1) data.f: data file name
##------------------------------------------------------------
hdpreg.fit <- function(data.y, data.x, data.study,
                       is.mdp=TRUE,
                       is.bin=FALSE,
                       scale.x=TRUE,
                       fit.rst.f=NULL,
                       ...) {
    f.read <- function(f) {
        rst <- data.matrix(read.table(f));
        unlink(f);
        rst
    }

    nstudies   <- length(unique(data.study));
    pt.default <- apply(data.x, 2, median);

    ##scale covariates for continuous
    x.scale <- rep(list(NULL), ncol(data.x));
    if (scale.x) {
        for (j in 1:ncol(data.x)) {
            lev.x <- length(unique(data.x[,j]));
            if (lev.x > 5) {
                ##continuous
                x.scale[[j]]$mean <- mean(data.x[,j]);
                x.scale[[j]]$sd   <- sd(data.x[,j]);
                data.x[,j]        <- scale(data.x[,j]);
            }
        }
    }

    ##------------FIT MODEL----------------------
    ##write init file and hdpmn
    hdpreg.WriteInitP(data.y, data.x, data.study, is.mdp, is.bin, ...);

    ##call C
    ##if (is.loaded("hdpreg")) {
    .C("hdpreg");
    ##} else {
    ##    system("../hdpreg");
    ##}

    ##organize fitting resuts
    fit.rst             <- list();
    fit.rst$"is.bin"    <- is.bin;
    fit.rst$"mdp.reg"   <- is.mdp;
    fit.rst$"nstudies"  <- nstudies;
    fit.rst$"x.scale"   <- x.scale;
    fit.rst$"pt.def"    <- pt.default;
    fit.rst$"beta"      <- f.read("beta.mdp");
    fit.rst$"ysig"      <- f.read("ysig.mdp");
    fit.rst$"nu"        <- f.read("nubeta.mdp");
    fit.rst$"sinv"      <- f.read("sbeta.mdp");

    if (is.mdp) {
        fit.rst$"par" <- f.read("par.mdp");
        fit.rst$"cn"  <- f.read("clustern.mdp");
        fit.rst$"mj"  <- f.read("mj.mdp");
    }

    ##------------SAVE RESULTS--------------------
    if (!is.null(fit.rst.f)) {
        save(fit.rst, file=fit.rst.f);
    }

    ##return
    class(fit.rst) <- "hdpfit";
    fit.rst
}

##predcition
hdpreg.pred <- function(pts=NULL, fit.rst=NULL, pred.rst.f=NULL, ...) {

    if (is.null(fit.rst)) return(NULL);

    if (is.null(pts)) {
        pts <- list(fit.rst$pt.def);
    }

    ##------------PREDICTION----------------------
    x.scale <- fit.rst$x.scale;
    pred    <- rep(list(NULL), length(pts));
    for (j in 1:length(pts)) {
        cur.covx  <- pts[[j]];
        ##scale covx
        for (k in 1:length(cur.covx)) {
            if (!is.null(x.scale[[k]])) {
                cur.covx[k] <- (cur.covx[k] - x.scale[[k]]$mean)/x.scale[[k]]$sd;
            }
        }
        pred[[j]] <- pred.R(cur.covx, fit.rst, ...);
    }

    pred.rst <- list(is.bin=fit.rst$is.bin,
                     nstudies=fit.rst$nstudies,
                     pts=pts,
                     pred=pred);

    ##------------SAVE RESULTS--------------------
    if (!is.null(pred.rst.f)) {
        save(pred.rst, file=pred.rst.f);
    }

    ##return
    class(pred.rst) <- "hdppred";
    pred.rst;
}

##------------------------------------------------------------
##               PLOT THE RESULTS
## Parameters
##
##------------------------------------------------------------
##draw densities for a single patient
plot.den.single <- function(lst.den.val, n.pt,
                            draw.bin=FALSE,
                            lens=NULL,
                            lcols=rep(c("black", "red", "green"),2),
                            ltys=rep(c(1,2),3),
                            quants=c(0.025, 0.975),
                            bin.label=TRUE,
                            x.lim=NULL) {

    lst.den <- lst.den.val[[1]];
    lst.val <- lst.den.val[[2]];

    ##number of studies+1
    n.studies <- length(lst.den)/n.pt;

    ##get min max of x and y in dens
    f.tmp <- function(cur.den) {
        max.y <- max(cur.den$y);
        min.x <- quantile(cur.den$x, quants[1]);
        max.x <- quantile(cur.den$x, quants[2]);
        rst   <- c(max.y, min.x, max.x);
        rst
    }

    lims <- NULL;
    for (i in 1:length(lst.den)) {
        cur.lim <- f.tmp(lst.den[[i]]);
        lims    <- cbind(lims, cur.lim);
    }

    max.y <- max(lims[1,]);
    if (is.null(x.lim)) {
        min.x <- min(lims[2,]);
        max.x <- max(lims[3,]);
    } else {
        min.x <- x.lim[1];
        max.x <- x.lim[2];
    }

    ##mains
    txt.main <- c(paste("Study", 1:(n.studies-1), sep=""), "Future");

    ##
    n.fig.row <- floor(sqrt(n.studies));
    n.fig.col <- ceiling(sqrt(n.studies));
    par(mfrow=c(n.fig.row, n.fig.col));
    for (i in 1:n.studies) {
        plot(NULL, ylim=c(0, 1.1*max.y), xlim=c(min.x, max.x),
             main=txt.main[i], xlab="Response", ylab="Density");

        ##draw binary
        if (draw.bin) {
            w.x   <- (max.x - min.x);
            w.l   <- 0.15;
            w.r   <- 0.35;
            w.bar <- (w.r-w.l)/n.pt;
            for (cov.j in 1:n.pt) {
                ey  <- mean(lst.val[[ (cov.j-1)*n.studies+i]]);
                ## print(paste("--study--", i,
                ##             "--pt--",    cov.j,
                ##             "--p(y=1)=", ey, sep=""));
                for (k in 0:1) {
                    c.y  <- (ey*k + (1-ey)*(1-k));
                    c.x1 <- min.x + w.x * (w.l+(cov.j-1)*w.bar+k*0.5);
                    c.x2 <- c.x1  + w.x * w.bar;
                    rect(c.x1, 0, c.x2, max.y*0.9*c.y,
                         col=lcols[cov.j], density=30, border="NA");
                    if (bin.label) {
                        text(c.x1+w.bar/2*w.x, max.y*0.9*c.y,
                             sprintf("%5.3f", c.y), col=lcols[cov.j], cex=1.2);
                    }
                }
            }
        }

        ##draw latent density
        for (cov.j in 1:n.pt) {
            lines(lst.den[[(cov.j-1)*n.studies+i]]$x,
                  lst.den[[(cov.j-1)*n.studies+i]]$y,
                  col=lcols[cov.j],  lty=ltys[cov.j], lwd=2);
        }

        ##add legend
        if (1 == i & !is.null(lens)) {
            legend("topleft",
                   legend=lens,
                   lty=ltys[1:length(lens)],
                   col=lcols[1:length(lens)], bty="n", cex=1.2);
        }

    }
}

## draw post figures of densities
plot.hdppred <- function(pred.rst,
                         fig.pts=NULL,
                         fig.f="pred",
                         fig.format="pdf",
                         fig.overlap=FALSE,
                         r=0, ...) {
    if (is.null(pred.rst)) return(NULL);

    ##plot selected patients
    if (!is.null(fig.pts)) {
        pred.rst$pts  <- pred.rst$pts[fig.pts];
        pred.rst$pred <- pred.rst$pred[fig.pts];
    }

    f.pt <- function(ps=NULL) {
        rst <- NULL;
        if (is.null(ps)) {
            ps <- 1:length(pred.rst$pts);
        }
        for (i in 1:length(ps)) {
            rst <- c(rst,
                     paste("x=(",
                           paste(pred.rst$pts[[i]], collapse=","),
                           ")",sep=""));
        }
        rst
    }

    ##r=0,1
    npts     <- length(pred.rst$pts);
    nstudies <- pred.rst$nstudies;
    pred     <- pred.rst$pred;
    cur.den  <- rep(list(NULL), npts*(nstudies+1));
    cur.val  <- rep(list(NULL), npts*(nstudies+1));

    inx <- 1;
    for (cov.j in 1:npts) {
        for (j in 1:(nstudies+1)) {
            cur.den[[inx]] <- density(pred[[cov.j]][[r+1]][[j]][,2]);
            cur.val[[inx]] <- pred[[cov.j]][[r+1]][[j]][,1];
            inx <- inx+1;
        }
    }

    if (fig.overlap) {
        ##overlap patients
        cur.fig.f <- paste(fig.f, "_pt_overlap.", fig.format, sep="");
        if ("ps" == fig.format) {
            postscript(cur.fig.f);
        } else {
            pdf(cur.fig.f);
        }
        plot.den.single(list(cur.den, cur.val), npts, pred.rst$is.bin, lens=f.pt(), ...);
        dev.off();
    } else {
        for (cov.j in 1:npts) {
            cur.fig.f <- paste(fig.f, "_pt_", cov.j, ".", fig.format, sep="");
            if ("ps" == fig.format) {
                postscript(cur.fig.f);
            } else {
                pdf(cur.fig.f);
            }
            plot.den.single(list(cur.den[(cov.j-1)*(nstudies+1)+(1:(nstudies+1))],
                                 cur.val[(cov.j-1)*(nstudies+1)+(1:(nstudies+1))]),
                            1,
                            pred.rst$is.bin,
                            lens=f.pt(cov.j),
                            ...);
            dev.off();
        }
    }
}

##------------------------------------------------------------
##               summerize hdpreg.pred
##------------------------------------------------------------
summary.single <- function(cur.pt, is.binary) {
    rnames <- c(paste("Study", 1:(length(cur.pt)-1)), "Future Study")
    tits   <- "Descriptive Statistics of Outcome:";
    if (is.binary) {
        rst.bin <- NULL;
        for (i in 1:length(cur.pt)) {
            rst.bin <- rbind(rst.bin, mean(cur.pt[[i]][,1]));
        }
        rst.bin <- cbind(rst.bin, 1-rst.bin);
        colnames(rst.bin) <- c("P(Y=1)", "P(Y=0)");
        rownames(rst.bin) <- rnames;
        print(tits);
        print(rst.bin, digits=4);
        tits <- "Descriptive Statistics of Latent Outcome:";
    }

    rst.y <- NULL;
    for (i in 1:length(cur.pt)) {
        cy  <- cur.pt[[i]][,2];
        c.r <- c(mean(cy), sd(cy), quantile(cy, c(0.025, 0.25, 0.5, 0.75, 0.975)));
        rst.y <- rbind(rst.y,
                       c(mean(cy), sd(cy), quantile(cy, c(0.025, 0.25, 0.5, 0.75, 0.975))));
    }
    colnames(rst.y) <- c("Mean", "SD", "2.5%", "25%", "Median", "75%", "97.5%");
    rownames(rst.y) <- rnames;
    print(tits);
    print(rst.y, digits=4);
}

summary.hdppred <- function(pred.rst, sum.pts=NULL, r=0) {
    pts    <- pred.rst[[3]];
    nstudy <- pred.rst[[2]];
    is.bin <- pred.rst[[2]];
    if (is.null(sum.pts)) {
        sum.pts <- 1:length(pts);
    }

    print(paste("The following prdicted outcome are based on clinical data from ",
                nstudy,
                " studies.",sep=""));

    if (1 == r) {
        print("The following prdicted outcome are study-specific only.");
    }

    for (i in 1:length(sum.pts)) {
        print(paste("Paitient ", sum.pts[i], " has covariates:", sep=""));
        print(pts[[sum.pts[i]]]);
        summary.single(pred.rst$pred[[sum.pts[i]]][[r+1]], is.bin);
    }
}
