\name{hdpreg.fit}
\alias{hdpreg.fit}
\title{Dirichlet Process Random Intercept model}
\description{Fits a Dirichlet Process Random Intercept regression model to data from multiple clinical studies.}
\usage{hdpreg.fit(data.y, data.x, data.study, mdp.reg=TRUE, fit.rst.f="regout.Rdata", ...)}
%- maybe also 'usage' for other objects documented here.

\arguments{
  \item{data.y}{patient response data (vector)}
  \item{data.x}{patient baseline covariates (matrix)}
  \item{data.study}{patient study index (vector)}
  \item{mdp.reg}{boolean. TRUE to fit a regression model with random intercept; FALSE to fit a regular regression model}
  \item{fit.rst.f}{file name to save the fitted result}
  \item{is.bin}{boolean. whether the response data is binary or not}
  \item{scale.x}{boolean. whether standardize continuous covariates}
  \item{n.iter}{number of posterior sampling iterations}
  \item{n.discard}{number of burn-in}
  \item{n.batch}{number of thinning}
  \item{verbose}{level of progress printed during program running. 0:no detailed progress printed. 3:most detailed progressprinted}
  \item{mcmc.eps}{whether to sample \eqn{\epsilon} or not}
  \item{eps}{initial value of \eqn{\epsilon}}
  \item{ae}{Beta parameter \eqn{a_e}}
  \item{be}{Beta parameter \eqn{b_e}}
  \item{pe1}{\eqn{\psi_1}}
  \item{pe0}{\eqn{\psi_0}}
  \item{m.prior}{whether to sample m}
  \item{B.prior}{whether to sample B}
  \item{S.prior}{whether to sample S}
  \item{alpha.prior}{whether to sample \eqn{M_k}}
  \item{za}{Gamma parameter \eqn{a_z}}
  \item{bz}{Gamma parameter \eqn{b_z}}
  \item{nu0}{prior mean of \eqn{\nu}}
  \item{R0}{prior variance of \eqn{\nu}}
  \item{q}{Wishart distribution parameter q}
  \item{R1}{Wishart distribution parameter \eqn{R_1}}
  \item{a0}{\eqn{a_0}}
  \item{b0}{\eqn{b_0}}
  \item{aB}{\eqn{a_B}}
  \item{bB}{\eqn{b_B}}
  \item{aa}{\eqn{a_M}}
  \item{ba}{\eqn{b_M}}
}

\details{ Consider \eqn{J} clinical studies. On study \eqn{j} (\eqn{j = 1, ..., J}), continuous
response, denoted by \eqn{z_{ij}}, is observed from the \eqn{i}th (\eqn{i=1,..., n_j}) patient with
baseline covariates \eqn{x_{ij}}. For binary responses, we denote them as \eqn{y_{ij}} and let
\eqn{z_{ij}} be the latent random variable which is unobserved but determines \eqn{y_{ij}} such that
\eqn{y_{ij}=1} if \eqn{z_{ij}>0} or \eqn{0} otherwise.

The model we consider is specified as follows.
  \eqn{z_{ij} \sim N(\theta_{ij} + \beta_j x_{ij}, \sigma_z^2)},
  \eqn{\theta_{ij} \sim N(\mu_{ij}, S) },
  \eqn{\mu_{ij}    \sim H_j },
  \eqn{H_j         = (1-\epsilon) F_0 + \epsilon F_j },
  \eqn{f_k(.| \mu_k, s_k, M_k) = \int \psi_{\mu, s} (.) d G_k(\mu, s)} for \eqn{0 \leq  k \leq J },
  \eqn{G_k       \sim DP(G, M_k) },
  \eqn{G         \sim N(m, B) }.

The priors are as follows.
  \eqn{\sigma_z^2  \sim IGamma(a_z=0.01, b_z=0.01) },
  \eqn{\beta_j   \sim N(\nu, S_\beta) },
  \eqn{\nu       \sim N(\nu_0=0, R_0=Diag(10^3)) },
  \eqn{S_\beta^{-1} \sim Wishart(q=10, R_1=Diag(0.1)) },
  \eqn{S      \sim IGamma(a_s=0.01, b_0=0.01)},
  \eqn{m         \sim N(a_0=0, b_0=10^3) },
  \eqn{B         \sim IGamma(a_B=0.01, b_B=0.01) },
  \eqn{M_k       \sim \Gamma(a_M=1, b_M=1)}.

We assign the following prior to the mixing parameter \eqn{\epsilon}:
\deqn{p(\epsilon) = \psi_0 \delta_0(\epsilon) + \psi_1 \delta_1(\epsilon) + (1 - \psi_0 - \psi_1) Beta(a_e=1, b_e=1),}
where \eqn{\delta_x(.)} is a point mass distribution at \eqn{x}.
}

\value{
A list with the follwing components:
 \item{nstudies }{
 number of studies
 }
 \item{pt.def}{
 default patient to be predicted which has the mean covariates
 }
 \item{x.scale}{mean and SD for continuous covariates}
 \item{beta}{
 posterior samples of \eqn{\beta}
 }
 \item{ysig}{
 posterior samples of \eqn{\sigma^2_z}
 }
 \item{nu}{
 posterior samples of \eqn{\nu}
 }
 \item{sinv}{
 posterior samples of \eqn{S_{\beta}^{-1}}
 }
 \item{cn}{
 posterior samples of number of components in each \eqn{F_j (j = 0, \ldots, J)}
 }
 \item{mj}{
 posterior samples of Dirichlet Process parameters
 }
 \item{par}{
 posterior samples of Dirichlet Process parameters
 }
}
\references{
MacEachern, S. N. & Muller, P. (1998), 'Estimating mixture of Dirichlet process models', Journal of Computational and Graphical Statistics 7(2), 223--238.

Lopes, H. F.; Muller, P. & Rosner, G. L. (2003), 'Bayesian Meta-analysis for Longitudinal Data Models Using Multivariate Mixture Priors', Biometrics 59(1), 66--75.

Muller, P. & Rosner, G. L. (1997), 'A Bayesian population model with hierarchical mixture priors applied to blood count data', Journal of the American Statistical Association 92(440), 1279--1292.}
