\name{hdpreg.pred}
\alias{hdpreg.pred}
\title{Predict patient outcome}
\description{Predict patient outcome with posterior samples of parameters from hepreg.fit}
\usage{hdpreg.pred(pts, fit.rst, pred.rst.f, ...)}
\arguments{
  \item{pts}{list of patients covariates to be predicted}
  \item{fit.rst}{list of results from hdpreg.fit}
  \item{pred.rst.f}{file name to save the prediction results}
  \item{sim.each}{number of predictions for each post sample from hdpreg.fit}
}
\value{
A list with the following components:
  \item{is.bin }{if the response \eqn{y} is binary}
  \item{nstudies }{number of studies}
  \item{pts}{patients covariates}
  \item{pred}{list of prediction results (list) for each patient. The prediction results are matrix with two columns. The first column is the predicted response \eqn{Y} and the second column is the predicted latent variable corresponding to response \eqn{Y}. In contionuous case, the two column are identical.}
}

\examples{
library("hdpreg")
data('DataRCT');
fit.rst  <- hdpreg.fit(DataRCT$Y, cbind(DataRCT$X1), DataRCT$Study,
                       is.mdp=TRUE, is.bin=FALSE,
                       n.iter=500,
                       n.discard=100);
pred.rst <- hdpreg.pred(fit.rst=fit.rst);
plot(pred.rst);
summary(pred.rst);
}
