/*============================================================
  Description:
      TO BE ADDED
=============================================================*/
#include "stdlib.h"
#include "math.h"
#include "stdio.h"

#include "interface.h"
#include "vector.h"
#include "matrix.h"
#include "rand.h"
#include "nrutil.h"
#include "bayes.h"
#include "regr.h"
#include "mess.h"
#include "ranlib.h"
#include "dcdflib.h"
#include "fortif.h"
#include "hdpreg.h"

/* #include "interface.c" */
/* #include "com.c" */
/* #include "vector.c" */
/* #include "matrix.c" */
/* #include "rand.c" */
/* #include "nrutil.c" */
/* #include "bayes.c" */
/* #include "regr.c" */
/* #include "mess.c" */
/* #include "ranlib.c" */
/* #include "dcdflib.c" */
/* #include "fortif.c" */
/* #include "ipmpar.c" */

/*================================================*/
/*                                                */
/*		 main                             */
/*                                                */
/*================================================*/


/**************************************************
                 global vars
**************************************************/
struct MDP_PARS mdp;
struct MCMC g;
struct Model mdl;
struct Data dta;
static int line=1; /* counts number of writes to output files */


/**************************************************
                 all MDP pars
**************************************************/
FILE *mfile, *parfile, *cnfile;

/*************************************************
                 all Regression Pars
**************************************************/
FILE *betafile, *ysigfile, *nubetafile, *sbetafile;

/*************************************************
                 Gibbs iterations
**************************************************/
int p, n_predupdate;
int n_class0, *member0, *r0; /* initial config */


/*===================================================*/
/*                  main                             */
/*===================================================*/
//int main() {
void hdpreg() {
    int j;
    //initialization
    init();
    mdp_init();

    //sampling
    for(g.iter=0;g.iter<g.niter;g.iter++){
	if (g.iter % 5 == 0) printf("\n %d:", g.iter);

        //if binary sample latent z put in mdp.y
        //otherwise put original dta.y to mdp.y
        if (1 == dta.bin) sample_latent();

	sample_beta();
        get_yinter();

        //sample V(y):needs to be before mdp precess
        sample_sigz2();

        if (1 == mdp.reg) {
	    mdp_gibbs();
	    if (g.sampleeps==1) {
		sample_eps();
	    }

	    //print out
	    if ((g.verbose>0) &
		(g.iter % g.npi == 0)) {
		printf(" %d:%d=", g.iter, mdp.n_class);
		for(j=0;j<=mdp.J;j++)
		    if (j==mdp.J)
			printf("%d",mdp.K[j]);
		    else
			printf("%d+",mdp.K[j]);
		printf("  ");
	    }
	}

        //regression
        //sample E(\beta)
        sample_nubeta();
        //sample V(\beta)
        sample_Sbeta();

	if (g.iter > g.ndiscard){
	    if (g.iter % g.npi == 0){
		mdp_writepars();
	    }
            /* update predictive */
	    if (g.iter % g.npredupdate == 0) {
		mdp_writepi();
                reg_writepi();
	    }
	}
    }
    printf("\n");

    //close files
    finish();
}


/*===========================================================*/
/*		 initialization & finish                     */
/*===========================================================*/

/*******************************************/
/*         init                            */
/*******************************************/
void init() {
    int i,j,k,s,currpa,pa,ni,npa;

    //read pars
    trim("init.giis","init-trim.giis");
    openIn("init-trim.giis");
    scanInt(" data-npatients ", &dta.npa);
    scanInt(" data-nstudies ", &dta.J);
    scanInt(" model-p1 ", &mdl.p1);
    dta.j = ivector(0, dta.npa);
    scanIntArray(" data-study ", dta.j, dta.npa);

    /* this is new compared to meta.c !! */
    scanInt(" data-N ",        &dta.N);
    scanInt(" data-bin ",      &dta.bin);
    scanInt(" mcmc-niter ",    &g.niter);
    scanInt(" mcmc-npi ",      &g.npi);
    scanInt(" mcmc-nbatch ",   &g.nbatch);
    scanInt(" mcmc-ndiscard ", &g.ndiscard);
    scanInt(" mcmc-verbose ",  &g.verbose);

    /* r.v. seeds */
    scanLong(" mcmc-seed1", &g.is1);
    scanLong(" mcmc-seed2", &g.is2);
    setseed(g.is1, g.is2);

    scanInt(" mcmc-eps ", &g.sampleeps);
    scanInt(" mcmc-npredupdate ", &g.npredupdate);
    scanDouble(" model-eps ",&mdl.eps);
    scanDouble(" model-ae ",&mdl.ae);
    scanDouble(" model-be ",&mdl.be);
    scanDouble(" model-pe0 ",&mdl.pe0);
    scanDouble(" model-pe1 ",&mdl.pe1);
    scanInt(" model-m_prior ", &mdl.m_prior);
    scanInt(" model-B_prior ", &mdl.B_prior);
    scanInt(" model-S_prior ", &mdl.S_prior);
    scanInt(" model-alpha_prior ", &mdl.alpha_prior);
    closeIn();

    //allocate memory
    dta.y       = dvector(0, dta.N-1);
    dta.x       = dmatrix(0, dta.npa-1, 0, mdl.p1-1);
    dta.n       = ivector(0, dta.J);

    //read data
    openIn("data-y.giis");
    scanDoubleArray(" data-y ", dta.y, dta.N);
    closeIn();
    openIn("data-x.giis");
    scanDoubleMatrix(" data-x ", dta.x, dta.npa, mdl.p1);
    closeIn();

    /* initialize hash table for first obs on patient i */
    for(i=0; i<dta.npa; i++){
	s = dta.j[i];
	dta.n[s] += 1;
    }

    /* check ni's */
    for(npa=0,j=1;j<=dta.J;j++)
	npa += dta.n[j];
    if (npa != dta.npa){ /* should never happen ... */
	printf("\n *** Error: Sum n[j] != npa.\n");
	exit(1);
    }
}

// finish mdp close files
void finish() {
    mdp_finish();
}


/***************************************************/
/*   init mdp                                      */
/***************************************************/
void mdp_init() {
    int i,j;
    double alpha;

    //read in sample size and dim
    trim("init.mdp","init-trim.mdp");
    openIn("init-trim.mdp");
    scanInt(" mdp-reg ", &mdp.reg);

    mdp.N = dta.npa;
    mdp.J = dta.J;
    mdp.y = dvector(0, mdp.N-1);
    y_assgn_x(mdp.y, dta.y, dta.npa);

    scanInt(" data-p ", &p);
    mdp.p = p;

    //allocate memory for pars & data
    mdp.betaj     = dmatrix(0, dta.J-1, 0, p-1);
    mdp.Yinter    = dvector(0, mdp.N-1);
    mdp.theta     = dvector(0, mdp.N-1);

    mdp.nu        = dvector(0, p-1);
    mdp.nu0       = dvector(0, p-1);
    mdp.R0        = dmatrix(0,p-1,0,p-1);
    mdp.R0_inv    = dmatrix(0,p-1,0,p-1);

    //variance for beta
    mdp.Sb_inv    = dmatrix(0,p-1,0,p-1);
    mdp.R1        = dmatrix(0,p-1,0,p-1);
    mdp.mu        = dvector(0, mdp.N-1);
    mdp.mj        = dvector(0, mdp.N-1);
    mdp.Vt        = dvector(0, mdp.N-1);

    mdp.alpha     = dvector(0,mdp.J);
    mdp.eta       = dvector(0,mdp.J);

    /* note: count, first & upd allow for one extra class = "new class" */
    mdp.count   = ivector(0,mdp.N);
    mdp.first   = ivector(0,mdp.N);
    mdp.member  = ivector(0,mdp.N-1);
    mdp.updated = ivector(0,mdp.N-1);
    mdp.prev    = ivector(0,mdp.N-1);
    mdp.next    = ivector(0,mdp.N);
    mdp.new     = ivector(0,mdp.N-1);
    mdp.r       = ivector(0,mdp.N-1);
    mdp.j       = ivector(0,mdp.N-1);
    mdp.K       = ivector(0,mdp.J);
    mdp.n       = ivector(0,mdp.J);
    mdp.qv      = dvector(0,mdp.N+1);

    i_zero(mdp.count,mdp.N+1);
    i_zero(mdp.n,mdp.J);
    i_zero(mdp.K,mdp.J);

    member0 = ivector(0,mdp.N-1);
    r0      = ivector(0,mdp.N-1);

    //------read in hyperpars and initial values
    scanDouble(" init-sig ", &mdp.sigz2);
    scanDouble(" par-az ", &mdp.az);
    scanDouble(" par-bz ", &mdp.bz);
    scanDoubleMatrix(" init-beta ", mdp.betaj, dta.J, p);
    scanDoubleArray(" par-nu0 ", mdp.nu, p);
    scanDoubleMatrix(" par-R0 ", mdp.R0, p, p);
    d_invert(mdp.p, mdp.R0, mdp.R0_inv);

    /* read and invert S */
    scanDoubleMatrix(" init-Sbinv ", mdp.Sb_inv, p, p);
    scanInt(" par-q ", &mdp.q);
    scanDoubleMatrix(" par-R1 ", mdp.R1, p, p);

    /* read m and B*/
    scanDouble(" init-m ", &mdp.m);
    scanDouble(" par-a0 ", &mdp.a0);
    scanDouble(" par-b0 ", &mdp.b0);

    scanDouble(" init-B ", &mdp.B);
    scanDouble(" par-aB ", &mdp.aB);
    scanDouble(" par-bB ", &mdp.bB);

    /* initialize mdp.St */
    mdp.St = mdp.sigz2 + mdp.B;

    scanDouble(" init-alpha ", &alpha);
    x_assgn_r(mdp.alpha, alpha, mdp.J+1);
    scanDouble(" par-aalpha ", &mdp.aalpha);
    scanDouble(" par-balpha ", &mdp.balpha);

    //initial config
    scanInt(" init-k0 ", &n_class0);
    if (n_class0 == 1)		/* one common cluster */
	for(i=0;i<mdp.N;i++){
	    member0[i] = 0;
	    r0[i] = 0;
	}
    else if (n_class0 == mdp.N)	/* N idiosync clusters */
	for(i=0;i<mdp.N;i++){
	    member0[i] = i;
	    r0[i] = 1;
	}
    else if (n_class0 == mdp.J){	/* J idiosync clusters */
	for(i=0;i<mdp.N;i++){
	    member0[i] = dta.j[i]-1;
	    r0[i] = 1;
	}
    }
    else if (n_class0 == mdp.J+1){ /* J idiosync clusters, 1 common cl */
	for(i=0;i<mdp.N;i++){
	    if (i%2 == 0){ /* common cluster */
		member0[i] = 0;
		r0[i] = 0;
	    } else { /* idiosync cluster */
		member0[i] = dta.j[i];
		r0[i] = 1;
	    }
	}
    }
    else{
	scanIntArray(" init-member0 ", member0, mdp.N);
	scanIntArray(" init-r0 ", r0, mdp.N);
    }
    closeIn();

    /* consistency check */
    if ((mdl.p1 != mdp.p) | (dta.npa != mdp.N) ){
	printf("\n*** inconsistent mdl.p1/mdp.p or dta.npa/mdp.N!!\n");
	exit(1);
    }

    mdp_initconfig();  //after initializing the data

    /* open files */
    mfile      = openOut("mj.mdp");
    parfile    = openOut("par.mdp");
    cnfile     = openOut("clustern.mdp");
    ysigfile   = openOut("ysig.mdp");
    betafile   = openOut("beta.mdp");
    nubetafile = openOut("nubeta.mdp");
    sbetafile  = openOut("sbeta.mdp");
}

/*************************************************************/
/*   initial configuration                                   */
/*************************************************************/
void mdp_initconfig(){
    int i, k;

    /* Note: values for aa and tau are required here! */
    mdp.n_class = 0;
    for (i = 0; i < mdp.N; i++){
	k = member0[i];
	if (mdp.count[k]==0) /* first obs in cluster */
	    mdp_newclass(i,r0[i]);
	else
	    mdp_addclass(i,k);
    }

    for (k=0; k<n_class0; k++){
	if (mdp.count[k]==0){
	    printf("\n *** Error: class %d has no members.\n",k);
	    exit(1);
	}
	mdp_setupmu(k,1);
    }

    mdp_checkclass();
}

/**************************************************/
/*                 mdp_finish                     */
/**************************************************/
void mdp_finish() {
    fclose(mfile);
    fclose(parfile);
    fclose(cnfile);
    fclose(ysigfile);
    fclose(betafile);
    fclose(nubetafile);
    fclose(sbetafile);
}

/*-----------------------------------------------------------*/
/*   check class for bogous classes                          */
/*-----------------------------------------------------------*/
void mdp_checkclass(){
    int i,n,k;

    for(k=0; k<mdp.n_class; k++){
	for(n=0, i=mdp.first[k]; i!=-1; n++, i = mdp.next[i]){
	    if ( n > mdp.count[k] )
		printf("\n*** Error: class %d is bogous (too big).\n", k);
	    if ( (mdp.j[k] > 0) & (dta.j[i] != mdp.j[k]) )
		printf("\n*** Error: class %d is bogous (inconsistent j).\n", k);
	    if ( (mdp.j[k] == 0) & (mdp.r[i] != 0) )
		printf("\n*** Error: class %d is bogous (inconsistent j/r).\n",k);
	}
	if (n != mdp.count[k])
	    printf("\n*** Error: class %d is bogous (too small).\n", k);
    }
}

/*-----------------------------------------------------------*/
/*   mdp_takeout                                             */
/*-----------------------------------------------------------*/
void mdp_takeout(int ind) {
  int k_old,i,j_old,k;

  /* take y[ind] out of it's current class */
  k_old = mdp.member[ind];
  j_old = mdp.j[k_old];
  mdp.updated[k_old] = 0;

  mdp.count[k_old] -= 1;
  mdp.n[j_old] -= 1;
  mdp.member[ind] = -1;
  if (mdp.prev[ind] == -1)
    mdp.first[k_old] = mdp.next[ind];
  else mdp.next[mdp.prev[ind]] = mdp.next[ind];
  mdp.next[ind] = -1;
  mdp.prev[ind] = -1;
  mdp.r[ind] = -1;

  /* if class k=k_old is empty */
  if (mdp.count[k_old] == 0){
    k = mdp.n_class-1;
    /* relabel class k=n_class-1 as k_old */
    mdp.count[k_old] = mdp.count[k];
    mdp.first[k_old] = mdp.first[k];
    mdp.j[k_old] = mdp.j[k];
    mdp.count[k] = 0;
    mdp.first[k] = -1;
    mdp.j[k] = -1;
    mdp.new[k_old] = mdp.new[k];

    mdp.mu[k_old] = mdp.mu[k];
    mdp.mj[k_old] = mdp.mj[k];
    //mdp.Vt_cd_inv[k_old] = mdp.Vt_cd_inv[k];
    mdp.Vt[k_old] = mdp.Vt[k];
    mdp.updated[k_old] = mdp.updated[k];

    /* update member-references to the relabeled class */
    for(i=mdp.first[k_old];i!=-1;i=mdp.next[i])
      mdp.member[i] = k_old;

    /* decrement n-class by one */
    mdp.n_class -= 1;
    mdp.K[j_old] -= 1;

    mdp_checkclass();
  }
}

/*------------------------------------------------------------*/
/*   mdp_addclass                                             */
/*   adds obs[ind] to class[k]                                */
/*------------------------------------------------------------*/
void mdp_addclass(int ind, int k) {
    int j;
    if (mdp.first[k]== -1) {
	error("add-class", "tried to add to empty class...", 1);
    }

    j = mdp.j[k];
    if ( (j > 0) & (j != dta.j[ind]) ){
	printf("\n *** Error: tried to add obs %d (j=%d) to class %d (j=%d).\n",
	       ind, dta.j[ind], k, j);
	exit(1);
    }

    mdp.n[j]        += 1;
    mdp.count[k]    += 1;
    mdp.member[ind]  = k;
    mdp.next[ind]    = mdp.first[k];
    mdp.first[k]     = ind;
    mdp.prev[mdp.next[ind]] = ind;
    mdp.prev[ind]    = -1;
    mdp.r[ind]       = (j==0) ? 0 : 1;

    mdp.updated[k] = 0;
    mdp_checkclass();
}

/************************************************************/
/*   new-class                                              */
/*   open a new class with y[ind] as first member           */
/************************************************************/
void mdp_newclass(int ind, int r)
{ int k, j;

  k = mdp.n_class;
  j = (r==0) ? 0 : dta.j[ind];

  /* update count and member */
  mdp.count[k] = 1;
  mdp.first[k] = ind;
  mdp.new[k] = g.iter;
  mdp.member[ind] = k;
  mdp.next[ind] = -1;
  mdp.prev[ind] = -1;
  mdp.r[ind] = r;
  mdp.j[k] = j;
  mdp.n_class += 1;
  mdp.K[j] += 1;
  mdp.n[j] += 1;
  mdp.updated[k] = 0;
  mdp_checkclass();
}

/* -----------------------------------------------------------
   mdp_ybar: in this case theta is the data
/* ----------------------------------------------------------- */
void mdp_ybar(int k, double *ybar) {
    int i;
    double nk;

    nk   = 1.0*mdp.count[k];
    *ybar = 0;
    for(i=mdp.first[k]; i!=-1; i=mdp.next[i]){
	*ybar += mdp.Yinter[i];
    }
    *ybar = *ybar / nk;
}


/**************************************************/
/*   mdp_setupmu                                  */
/**************************************************/
void mdp_setupmu(int k, int draw) {
    double ybar, nk;
    double Vj;
    double stdn;

    if (mdp.count[k] == 0){
	printf("\n*** tried to update mu for empty class (k=%d)\n",k);
	exit(1);
    }

    /* get pars for conditional posterior p(mu|...) */
    mdp_ybar(k, &ybar);
    nk = 1.0*mdp.count[k];

    /* compute posterior mean cov matrix for mu[k] */
    nn_bayes1(1.0, 1.0/mdp.B, mdp.m, 1.0/nk, 1.0/mdp.sigz2, ybar,
	      &Vj, &mdp.mj[k]);

    if (isnan(mdp.mj[k])) {
	printf("\n*** Error mj[%d] is nan \n", k);
	exit(1);
    }

    /* if draw, then sample mu[k] */
    if (draw==1) {
	stdn      = norm_rand();
	mdp.mu[k] = mdp.mj[k] + sqrt(Vj) * stdn;
    }

    mdp.Vt[k]      = Vj + mdp.sigz2;
    mdp.updated[k] = 1;
}


/*************************************************************/
/*   MDP Gibbs Sampler                                       */
/*************************************************************/
void mdp_gibbs() {
    int start,k;

    //if (mdl.S_prior) mdp_sampleS();
    mdp_sampleconfig();
    for(k=0; k<mdp.n_class; k++){
	mdp_setupmu(k,1);
    }
    if (mdl.alpha_prior) mdp_samplealpha();
    if (mdl.B_prior) mdp_sampleB();
    if (mdl.m_prior) mdp_samplem();
}

/***********************************************************/
/*   sample config                                         */
/***********************************************************
   does the Gibbs over configurations
   Pr(y[i] in class j | ...) proportional to
   for j=0,..n_class-1:
      count[j]*MV-N(y[i]; mut[j], Vt[j])
      mut[j] and Vt[j] computed in make_mut_Vt
   for j=n_class (i.e. new class): same expression with
                                   count[n_class] = alpha
************************************************************/
void mdp_sampleconfig() {
    int    ji,     jl,  l,  k, k_new, i, kk;
    double *alpha, eps, kd, u, py;

    kd    = 1.0*mdp.n_class;
    alpha = mdp.alpha;
    eps   = mdl.eps;

    for (i = 0; i < mdp.N; i++){
	mdp_takeout(i);
        x_zero(mdp.qv, mdp.n_class+2);
        
        /* compute prob pi[i] = pi*[k]       */
	for(l=0; l<mdp.n_class; l++){
	    jl = mdp.j[l];
	    if (jl == dta.j[i]){ /* idiosync cluster of same  Fj */
		if (0 == mdp.updated[l]) {
		    mdp_setupmu(l,0);
		}

		mdp.qv[l] = mdp.count[l] * eps/(alpha[jl]+mdp.n[jl])*
                    pdfnorm(mdp.Yinter[i], mdp.mj[l], sqrt(mdp.Vt[l]));
	    } else if (jl == 0) {   /* other common cluster */
		if (0 == mdp.updated[l]) {
		    mdp_setupmu(l,0);
		}

		mdp.qv[l] = mdp.count[l] * (1-eps)/(alpha[0]+mdp.n[0])*
                    pdfnorm(mdp.Yinter[i], mdp.mj[l], sqrt(mdp.Vt[l]));
	    }
	}

	/* Prob of new class */
	/* qv[n_class = alpha * MV-N(y[i],m,V+tau*B)*/
	py = pdfnorm(mdp.Yinter[i], mdp.m, sqrt(mdp.sigz2+mdp.B));

        /* new idiosync cluster */
	ji = dta.j[i];
	mdp.qv[mdp.n_class] = alpha[ji]*eps/(alpha[ji]+mdp.n[ji])*py;
	
        /* new common cluster */
	mdp.qv[mdp.n_class+1] = alpha[0]*(1-eps)/(alpha[0]+mdp.n[0])*py;

	/* draw the index of the new class for ind */
	multinomial(1,mdp.n_class+2,mdp.qv,&k_new);     
    
	if (k_new == mdp.n_class){ /* create a new idiosync class */
	    mdp_newclass(i,1);
	} else if (k_new == mdp.n_class+1){ /* create a new common class */
	    mdp_newclass(i,0);
	} else{
	    /* add to existing class */
	    mdp_addclass(i,k_new);
	}
    }
    mdp_checkclass();
}

/******************************************************/
/*   sample_latent: generate latent Z                 */
/*                  associated with binary Y          */
/******************************************************/
void sample_latent() {
    int i, j, k;
    double cut, m, s, tmp;

    s   = mdp.sigz2;
    cut = 0.0;
    for (i=0; i<dta.npa; i++) {
        j = dta.j[i];
        m = 0;
        //x*beta
	for (k = 0; k < mdp.p; k++) {
	    m += mdp.betaj[j-1][k] * dta.x[i][k];
	}

        //mdp intercept
	if (1 == mdp.reg) {
	    m += mdp.mu[mdp.member[i]];
	}

       	if (dta.y[i]==1.0) {
	    tmp = norm_trunc_left(m, s, cut);
	}
	else {
	    tmp = norm_trunc_right(m, s, cut);
	}
	mdp.y[i] = tmp;
    }
}


/***********************************************************/
/*               sample alpha                              */
/***********************************************************
   draw alpha and eta from:
   alpha from     pi*Gamma(a0+n_class, b0-log(eta)) +
              (1-pi)*Gamma(a0+n_class-1, b0-log(eta)),
         where pi = (a+n-class-1)/(a0+n_class-1 + n*b0 - n*log(eta)).
   eta   from Beta(alpha+1,n).
*/
void mdp_samplealpha() {
    /* parameters in Gamma call */
    double a,  b, pi;
    double u;
    int j;

    for (j=0;j<=mdp.J;j++){
	if (mdp.n[j]==0){ /* no obs alloc to measure j */
	    a = mdp.aalpha;
	    b = mdp.balpha;
	} else {
	    /* draw eta */
	    mdp.eta[j] = betadev(mdp.alpha[j]+1.0, mdp.n[j]*1.0);

	    /* compute pars for gammas for alpha */
	    b  = mdp.balpha - log(mdp.eta[j]);
	    pi = (mdp.aalpha + mdp.K[j] - 1)/
		(mdp.aalpha + mdp.K[j] -1 + mdp.n[j]*(mdp.balpha - log(mdp.eta[j])));
	    duniform(1,&u);
	    a = (u<pi) ? mdp.aalpha+mdp.K[j] : mdp.aalpha+mdp.K[j]-1;
	    /* draw alpha with prob pi from Gamma(a1,b) else Gamma(a2,b) */
	}

	mdp.alpha[j] = gamdev_ab(a,b);
    }
}

/************************************************************/
/*                    draw B                                */
/* sample B from p(B|..) =
   W(1/B; k+c, [cC + 1/tau*sum (mu[k]-m)(mu[k]-m)']^-1)
*/
/***********************************************************/
void mdp_sampleB() {
    double Sk, S1, z, binv;
    int k;

    /* Sk = sum (mu[k]-m)(mu[k]-m)' */
    Sk=0;
    for(k=0; k<mdp.n_class; k++){
	z   = mdp.mu[k] - mdp.m;
	Sk += z*z;
    }

    /* S1 = s*S+Sk */
    binv  = gamdev_ab(mdp.aB + mdp.n_class/2, mdp.bB + Sk/2);
    mdp.B = 1/binv;

    /* error check */
    if (isnan(mdp.B) | isinf(mdp.B)) {
	error("mdp_sampleB", "B is NaN or Inf.", 1);
    }
}

/************************************************************/
/*               sample m                                   */
/*                                                          */
/* m from p(m|..) = N(m; m(mubar), T)                       */
/* m(ybar[k]) = T*(B-inv/tau*k*mubar + A-inv*a)             */
/* T-inv = B-inv/tau*k + A-inv                              */
/*************************************************************/
void mdp_samplem() {
    double mubar, K;
    int i;

    K = 1.0*mdp.n_class; /* need double */

    /* make mubar = 1/K sum mu[k] */
    mubar = 0;
    for(i=0; i<mdp.n_class; i++){
	mubar += mdp.mu[i];
    }
    mubar = mubar/K;

    /* compute and sample posterior on m */
    nn_bayes1_rand(1.0, 1.0/mdp.b0, mdp.a0, 1.0/K, 1.0/mdp.B, mubar,
                   &mdp.m);

    if (isnan(mdp.m)) {
	error("mdp_samplem", "mdp.m is NaN.", 1);
    }
}

/**************************************************
   sample_eps
  samples eps from conditional posterior
     p(eps|..) = 0 with prob c*pi0*I(n0=N)
               = 1 with prob c*pi1*I(n1=N)
	                                            B(ae*,be*)
               ~ Be(ae*,be*), with pr c*(1-pi0-pi1)*----------
	                                            B(ae, be )
     where ae*=ae+n1, be*=be+n0, n1=Sum r[ji], n0=N-n1
************************************************* */
void sample_eps() {
    int N,n1,n0,i,j,k;
    double q[3], ae1,be1,ae,be;

    N = dta.npa;
    /* compute n0, n1 */
    n1 = 0;
    for(i=0; i<N; i++) {
	n1 += mdp.r[i];
    }
    n0 = N-n1;

    /* compute q0,q1,q2 */
    ae = mdl.ae;
    be = mdl.be;
    mdl.ae1 = ae1 = ae+n1;
    mdl.be1 = be1 = be+n0;
    if ( (n0==N) | (n1==N) ){
	q[0] = (n0==N) ? mdl.pe0 : 0.0;
	q[1] = (n1==N) ? mdl.pe1 : 0.0;
	q[2] = (1-mdl.pe0-mdl.pe1)*
	    exp( lgamma(ae1)+lgamma(be1)-lgamma(ae1+be1)-    /* B(ae1,be1) */
		 (lgamma(ae )+lgamma(be )-lgamma(ae +be ))); /* B(ae, be ) */
	multinomial(1,3,q,&k);

        if (k==0)
	    mdl.eps = 0;
	else if (k==1)
	    mdl.eps = 1;
	else
	    mdl.eps = betadev(ae1,be1);
    } else {
	mdl.eps = betadev(ae1,be1);
    }
}
/*-------------------------------------------------*/
/*    sample beta                                  */
/*-------------------------------------------------*/
void sample_beta() {
    int j;
    for (j=1; j <= dta.J; j++) {
	sample_beta_j(j, mdp.betaj[j-1]);
    }
}

//sample beta for an individual study
void sample_beta_j(int j, double* beta) {
    int nj, i, k, tmpi, p;
    double *yj, **xj,  **xx, *xy;
    double *num, **Sm, **Sm_inv, *tmpnum;

    //number of pts in study j
    nj  = dta.n[j];
    p   = mdp.p;

    //allocate memory
    yj     = dvector(0, nj-1);
    xj     = dmatrix(0, nj-1, 0, p-1);
    xx     = dmatrix(0, p-1, 0, p-1);
    xy     = dvector(0, p-1);
    num    = dvector(0, p-1);
    tmpnum = dvector(0, p-1);
    Sm     = dmatrix(0, p-1, 0, p-1);
    Sm_inv = dmatrix(0, p-1, 0, p-1);

    //get yj xj
    tmpi = 0;
    for (i=0; i<dta.npa; i++) {
	if (j == dta.j[i]) {
	    yj[tmpi] = mdp.y[i] - mdp.mu[mdp.member[i]];
            for (k=0; k < p; k++) {
		xj[tmpi][k] = dta.x[i][k];
	    }
	    tmpi++;
	}
    }

    if (tmpi != nj) {
	printf("\n *** Error: n[j] != npa[j]. \n");
    }

    //get x'x and x'y
    AtB(xj, nj, mdp.p, xj, nj, mdp.p, xx);
    Rtx(xj, nj, mdp.p, yj, xy);

    //nu_m sigma_m
    rA_plus_sB(1.0, mdp.Sb_inv, 1.0/mdp.sigz2, xx, Sm_inv, mdp.p);
    d_invert(mdp.p, Sm_inv, Sm);
    rx(1.0/mdp.sigz2, xy, mdp.p);
    Rx_plus_y(mdp.Sb_inv, mdp.nu, xy, tmpnum, mdp.p, mdp.p);
    Rx(Sm, mdp.p, mdp.p, tmpnum, num);

    //beta ~ MVN(num, Sm)
    mvnS_rand(mdp.p, num, Sm, beta);

    //free memory
    free_dvector(yj, 0, nj-1);
    free_dmatrix(xj, 0, nj-1, 0, p-1);
    free_dmatrix(xx, 0, p-1, 0, p-1);
    free_dvector(xy, 0, p-1);
    free_dvector(num, 0, p-1);
    free_dvector(tmpnum, 0, p-1);
    free_dmatrix(Sm, 0, p-1, 0, p-1);
    free_dmatrix(Sm_inv, 0, p-1, 0, p-1);
}

/*-------------------------------------------------*/
/*    get Yintercept = Y - X*Beta_j                */
/*-------------------------------------------------*/
void get_yinter() {
    int i, j, k;

    for (i=0; i < dta.npa; i++) {
        //study
        j = dta.j[i];
        mdp.Yinter[i] = mdp.y[i];
        for (k = 0; k < mdp.p; k++) {
            //x*beta
	    mdp.Yinter[i] -= mdp.betaj[j-1][k] * dta.x[i][k];
	}
    }
}

/*-------------------------------------------------*/
/*    sample nu = E(\beta_j)                       */
/*-------------------------------------------------*/
void sample_nubeta() {
    int j, p;
    double *betabar;
    //, *mum, **rm, **rm_inv, *z1, *z2;

    p = mdp.p;

    //allocate memory
    betabar = dvector(0, p-1);

    //get betabar
    for  (j=0; j<dta.J; j++) {
	x_plus_y(betabar, mdp.betaj[j], betabar, p);
    }
    rx(1.0/dta.J, betabar, p);

    nn_bayes_rand(1.0, mdp.R0_inv, mdp.nu0, 1.0/dta.J, mdp.Sb_inv, betabar, mdp.nu, p);

    //free memory
    free_dvector(betabar, 0, p-1);
}

/*-------------------------------------------------*/
/*    sample Sbeta = var(\beta_j)                  */
/*-------------------------------------------------*/
void sample_Sbeta() {
    int j, p;
    double **Stheta, *z, **sm_cd_inv;

    p = mdp.p;

    //allocate memory
    Stheta    = dmatrix(0, p-1, 0, p-1);
    z         = dvector(0, p-1);
    sm_cd_inv = dmatrix(0, p-1, 0, p-1);

    //compute S_\theta
    for(j=0; j < dta.J; j++) {
	x_min_y(mdp.betaj[j], mdp.nu, z, p);
	A_plus_rxxt(Stheta, 1.0, z, Stheta, p);
    }

    //S0+Stheta
    rA_plus_sB(1.0, mdp.R1, 1.0, Stheta, Stheta, p);
    cdinv(p, Stheta, sm_cd_inv);

    //sample S and S_inv
    wishart(mdp.q+dta.J, p, sm_cd_inv, mdp.Sb_inv);

    //free memory
    free_dmatrix(Stheta, 0, p-1, 0, p-1);
    free_dmatrix(sm_cd_inv, 0, p-1, 0, p-1);
    free_dvector(z, 0, p-1);
}


/*-------------------------------------------------*/
/*    sample \sigma_z^2                            */
/*-------------------------------------------------*/
void sample_sigz2() {

    if (1 == dta.bin) {
	mdp.sigz2 = 1;
	return;
    }

    int i, j, k;
    double cur, ssr;
    double siginv;

    ssr = 0;
    for (i=0; i<dta.npa; i++) {
        cur  = mdp.Yinter[i] - mdp.mu[mdp.member[i]];
	ssr += cur * cur;
    }

    //sample from gamma az+dta.npa/2, bz+ssr/2
    siginv    = gamdev_ab(mdp.az + dta.npa/2, mdp.bz + ssr/2);
    mdp.sigz2 = 1/siginv;
}

/**************************************************/
/*  mdp_writepars                                 */
/**************************************************/
void mdp_writepars() {
    int i,j;
    double tmp;

    if (0 == mdp.reg) {
	return;
    }

    line += 1;
    if (line % 100 == 0){
	fclose(parfile);
	parfile = openAppend("par.mdp");
    }

    fprintf(parfile,"%4d %7.5e %7.5e %7.5e %7.5e", g.iter, mdl.eps, mdp.S, mdp.m, mdp.B);
    for(j=0; j<=mdp.J; j++) {
	fprintf(parfile," %7.5e ", mdp.alpha[j]);
    }
    fprintf(parfile,"\n");

    /* write no cluster for F_0 and each F_J*/
    fprintf(cnfile, "%d", g.iter);
    for(j=0; j<=mdp.J; j++) {
	fprintf(cnfile, " %d", mdp.K[j]);
    }
    fprintf(cnfile, "\n");
}

/**************************************************/
/*          write pars                            */
/**************************************************/
void reg_writepi() {
    int j,k;
    if (line % 100 == 0){
	fclose(ysigfile);
        fclose(betafile);
        fclose(nubetafile);
        fclose(sbetafile);

	ysigfile   = openAppend("ysig.mdp");
        betafile   = openAppend("beta.mdp");
        nubetafile = openAppend("nubeta.mdp");
        sbetafile  = openAppend("sbeta.mdp");
    }

    /* write ysig */
    fprintf(ysigfile, "%d %6.4f \n", g.iter, mdp.sigz2);

    /* write beta */
    fprintf(betafile, "%d", g.iter);
    for (j=0; j < dta.J; j++) {
	for (k=0; k < mdp.p; k++) {
	    fprintf(betafile," %7.5e ",mdp.betaj[j][k]);
	}
    }
    fprintf(betafile, "\n");

    /*write e(beta)*/
    fprintf(nubetafile, "%d", g.iter);
    for (k=0; k < mdp.p; k++) {
	fprintf(nubetafile," %7.5e ",mdp.nu[k]);
    }
    fprintf(nubetafile, "\n");

    /*write inverse var(beta)*/
    fprintf(sbetafile, "%d", g.iter);
    for (j=0; j < mdp.p; j++) {
	for (k=0; k < mdp.p; k++) {
	    fprintf(sbetafile," %7.5e ",mdp.Sb_inv[j][k]);
	}
    }
    fprintf(sbetafile, "\n");

}

//write mdp parameters
void mdp_writepi() {
    int i,j,k,jj;
    double w,eps,*alpha, sig, ejj;

    if (0 == mdp.reg) {
	return;
    }

    eps   = mdl.eps;
    alpha = mdp.alpha;

    if (line % 100 == 0){
	fclose(mfile);
	fclose(cnfile);

	mfile    = openAppend("mj.mdp");
	cnfile   = openAppend("clustern.mdp");
    }

    for (jj=0;jj<=mdp.J+1;jj++){
	if (jj==mdp.J+1)
	    w = 1;
	else
	    w = alpha[jj]/(alpha[jj]+mdp.n[jj]);

	//eps or 1-eps
        ejj = (0 == jj) ? (1-eps) : eps;
	/* write m */
	fprintf(mfile, "%d %d %7.5e %7.5e %8.4f %8.4f \n", g.iter, jj, ejj, w, mdp.m, mdp.B);
    }

    for(j=0; j<mdp.n_class; j++){
	jj = mdp.j[j];
	w  = mdp.count[j]/(alpha[jj]+mdp.n[jj]);

	//eps or 1-eps
        ejj = (0 == jj) ? (1-eps) : eps;

	/* write m */
        //Vt - sigz2 = Vj
	fprintf(mfile,"%d %d %7.5e %7.5e %8.4f %8.4f\n",
		g.iter, jj, ejj, w, mdp.mj[j], mdp.Vt[j]-mdp.sigz2);
    }
}
